import requests
import bs4
from datetime import datetime, timedelta
import matplotlib.pyplot as plt
from typing import Any

class StockData:
    """
    This represents a stock

    Useful attributes:
    - short_code
    - url: Boursorama URL
    - soup_res: Beautiful_Soup result
    - name: Name of the stock
    - ISIN: ISIN code of the stock
    - price: price of the stock currently
    - DER: Last of the day
    - OUV: Opening of the day
    - highest: Highest of the day
    - lowest: Lowest of the day
    - Volume: Volume of the day

    Useful method:
    plot_all

    """
    def __init__(self, short_code: str):
        """
        The StockData class

        :param short_code: code of the Boursorama format.
        Example: 1rPASY

        """
        self.short_code = short_code
        BASE_BOURSO_LINK = "https://www.boursorama.com/cours/"
        # PARSING
        self.url = BASE_BOURSO_LINK + self.short_code
        res = requests.get(self.url,
                           headers={'User-Agent': 'Mozilla/5.0'})
        # Checking for Bad download
        try:
            res.raise_for_status()
        except Exception as exc:
            raise TimeoutError("There was a problem: %s" % (exc))
        soup_res = bs4.BeautifulSoup(res.text, 'html.parser')
        self.soup_res = soup_res
        self._get_data_from_soup(soup_res)


    def _get_data_from_soup(self, soup_res):
        # name c-faceplate__company-link
        self.name = soup_res.find('a', {
            'class': 'c-faceplate__company-link'}).contents[0].split("\n")[
            1].strip()
        # price c-instrument c-instrument--last
        price_str = soup_res.find('span', {
            'class': 'c-instrument c-instrument--last'}).contents[0]
        self.price = float(price_str.replace(",", "."))

        # isin code c-faceplate__isin
        self.ISIN_code, self.short_code = soup_res.find('h2', {
            'class': 'c-faceplate__isin'}).contents[0].split(" ")

        # tables
        tables = soup_res.find("table",
                               {"class": "c-table c-table--generic"})
        head_table = self._find_head(tables)
        data_table = self._find_data(tables, len_max=len(head_table))
        self.head_table = head_table
        self.data_table = data_table
        assert len(data_table) == 5
        self.DER = [data_table[0], head_table]
        self.OUV = [data_table[1], head_table]
        self.highest = [data_table[2], head_table]
        self.lowest = [data_table[3], head_table]
        self.volume = [data_table[4], head_table]

    def _find_head(self, table_in):
        L = []
        CURRENT_YEAR = datetime.now().year
        for head_i in table_in.find_all("th"):
            date_bs = head_i.find("span", {"aria-hidden":"true"})
            if date_bs is None:
                continue

            day, month = date_bs.contents[0].split("-")
            day_dt = datetime(year=CURRENT_YEAR, month=int(month), day=int(day))
            L.append(day_dt)
        return L

    def _find_data(self, table_in, len_max):
        L = []
        tables_i = table_in.find_all('td', {
            'class': 'c-table__cell c-table__cell--dotted c-table__cell--tiny'})
        L2 = []
        for str_i in tables_i:
            value_str = str_i.contents[0].split("\n")[1].strip()
            value_float = float(value_str.replace(",", ".").replace(" ", ""))
            L2.append(value_float)
            if len(L2) >= len_max:
                L.append(L2)
                L2 = []
        return L


    def plot(self, index_data=0):
        label_list = ["LAST OF DAY", "OPENING OF DAY", "HIGHEST OF DAY", "LOWEST OF DAY", "VOLUME"]
        label = label_list[index_data]
        data_table = self.data_table[index_data]
        plt.plot(self.head_table, data_table, label=label)
        plt.title(label)
        plt.show()

    def plot_all(self):
        label_list = ["LAST OF DAY", "OPENING OF DAY", "HIGHEST OF DAY", "LOWEST OF DAY"]
        hour_shift = {
            0: timedelta(hours=17, minutes=30), # Closing at 17h30
            1: timedelta(hours=9), # Morning, opening at 9h
            2: timedelta(hours=14), # Mid-day, opening at 14h
            3: timedelta(hours=14), # Mid-day, opening at 14h
        }


        for i in range(len(label_list)):
            time_serie = [k+hour_shift[i] for k in self.head_table]
            plt.scatter(time_serie, self.data_table[i], label=label_list[i])
        plt.title(f"{self.name}/ISIN:{self.ISIN_code}")
        plt.legend()
        plt.show()


    @classmethod
    def find_by_search_engine(cls,
                     query: str) -> Any: # Self:
        """
        Finds a stock using the Boursorama search engine
        :param query: string of the query
        :return: Self
        """
        url_find = f"https://www.boursorama.com/recherche/?query={query}&seeAllResults=tous-les-resultats"
        res = requests.get(url_find,
                           headers={'User-Agent': 'Mozilla/5.0'})
        # Checking for Bad download
        try:
            res.raise_for_status()
        except Exception as exc:
            raise TimeoutError("There was a problem: %s" % (exc))
        soup_res = bs4.BeautifulSoup(res.text, 'html.parser')
        soup_res = soup_res.find("tbody", {"class": "c-table__body"})
        soup_res = soup_res.find("tr")
        stock_code = soup_res["data-ist"]
        print(f"Found {stock_code} for query {query}")
        return cls(stock_code)



import requests
import bs4
from dataclasses import dataclass
from typing import Union, Literal
from tqdm import tqdm


@dataclass
class Entry:
    """
    Represents a Boursorama recommendation entry
    """
    name: str
    advice : str
    last: Union[str, float]
    obj_100d : float
    potential: Union[str, float]
    nb_analysts : str
    BNA_year: float
    rend_year: Union[str, float]
    per_year: Union[str, float]
    per_last_year: Union[str, float]


class ForecastsBourso:
    """
    Object to handle Boursorama recommendations
    Useful methods:
    forward_potential
    backward_potential
    """
    def __init__(self):
        self.list_entry = []
        self.set_gathered_stocks = set()
        prefix_url = "https://www.boursorama.com/bourse/actions/consensus/recommandations-paris/"
        postfix_url = "?national_market_filter%5Bmarket%5D=SRD&national_market_filter%5Bsector%5D=&national_market_filter%5Banalysts%5D=1&national_market_filter%5Bperiod%5D=2024&national_market_filter%5Bfilter%5D=&sortColumn=consPotential&orderAsc=1"

        N_max = 20
        list_urls = [prefix_url+postfix_url] + [prefix_url+f"page-{k}"+postfix_url for k in range(1, N_max+1) if k != 1]
        res_end = None
        for url_i in tqdm(list_urls):
            if res_end == "End reached":
                break
            res = requests.get(url_i,
                               headers={'User-Agent': 'Mozilla/5.0'})
            # Checking for Bad download
            try:
                res.raise_for_status()
            except Exception as exc:
                raise TimeoutError("There was a problem: %s" % (exc))
            soup_res = bs4.BeautifulSoup(res.text, 'html.parser')
            self.soup_res = soup_res
            soup_2 = soup_res.find("table", {"class": "c-table c-table--generic c-table--generic c-shadow-overflow__table-fixed-column"}).find("tbody")
            check_in = 0
            for soup_i in soup_2.find_all("tr", {"class": "c-table__row"}):
                res_end = self._parse_entry(soup_i, check_in=(check_in>4))
                if res_end == "End reached":
                    break
                check_in += 1


    @property
    def forward_potential(self):
        return reversed(self.list_entry)

    @property
    def backward_potential(self):
        return self.list_entry

    def _parse_entry(self, soup_i, check_in) -> Union[None, Literal["End reached"]]:
        name = soup_i.find("a", {"class": "c-link c-link--animated"}).contents[0]
        if name in self.set_gathered_stocks:
            if check_in:
                return "End reached"
            else:
                return None
        else:
            self.set_gathered_stocks.add(name)
        advice = soup_i.find("span", {"class": "u-only-clipboard"}).contents[0]
        L = []
        for data_i in soup_i.find_all("td"):
            content = str(data_i.contents[0])
            if content is None:
                continue
            content = content.strip()
            if "c-table__cell--negative" in data_i["class"]:
                if content not in ["-", "\n-"]:
                    #print(content)
                    content = "-" + content
            L.append(content)
        L = L[2:]
        last = self._format_to_float(L[0])
        obj_100d = self._format_to_float(L[1])
        potential = self._format_to_float(L[2])
        nb_analysts = L[3]
        BNA_year = self._format_to_float(L[4])
        rend_year = self._format_to_float(L[5])
        per_year = self._format_to_float(L[6])
        per_last_year = self._format_to_float(L[7])
        new_entry = Entry(
            name=name,
            advice=advice,
            last=last,
            obj_100d=obj_100d,
            potential=potential,
            nb_analysts=nb_analysts,
            BNA_year=BNA_year,
            rend_year=rend_year,
            per_year=per_year,
            per_last_year=per_last_year
        )
        self.list_entry.append(new_entry)
        return None

    def _format_to_float(self, string_in):
        if string_in in ["Atteint", "-"]:
            return string_in
        percent = False
        if "%" in string_in:
            string_in = string_in.replace("%", "")
            percent = True
        string_in = string_in.replace(",", ".").replace(" ", "").replace("USD", "").replace("EUR", "").replace("GBP", "")
        fl = float(string_in)
        if percent:
            fl = fl/100
        return fl

if __name__ == "__main__":
    ForecastsBourso()


from setuptools import setup, find_packages

setup(
    name='boursocrawling',
    version='1.0.0',
    author='Alexandre Delaisement',
    author_email='',
    description='A module for getting Boursorama Data',
    long_description='A module to get Boursorama stock data, recommendation, and search engine.',
    long_description_content_type='text/markdown',
    url='https://github.com/AlexandreDela/boursocrawling',
    packages=['boursocrawling'],
      package_dir={'boursocrawling': 'src/boursocrawling'},
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Financial and Insurance Industry',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Programming Language :: Python :: 3.12',

    ],
    keywords='finance',
    python_requires='>=3.8',
    install_requires=[
        "bs4",
        "numpy>=1.21.0",
        "matplotlib>=3.3.0",
        "requests",
        "pytest >= 8.1.1"
    ],
    project_urls={
        'Source': 'https://github.com/AlexandreDela/boursocrawling',
    },
)